import { useContext, useDebugValue, useEffect, useRef, useState } from 'react';
import { NoStoreError } from './error';
import { defaultStoreValue, getStoreContext } from './store';
export function Consumer(props) {
    const store = useStore(props.of);
    return props.children(store);
}
export function useStore(s, deps) {
    const name = s.displayName || s.name;
    useDebugValue(name);
    const Context = getStoreContext(s);
    const container = useContext(Context);
    const [state, setState] = useState(container.state);
    const depsRef = useRef([]);
    useEffect(() => {
        const subscriber = () => {
            if (!deps) {
                setState(container.state);
            }
            else {
                const oldDeps = depsRef.current;
                const newDeps = deps(container.state);
                if (compare(oldDeps, newDeps)) {
                    setState(container.state);
                }
                depsRef.current = newDeps;
            }
        };
        container.subscribers.add(subscriber);
        return () => {
            container.subscribers.delete(subscriber);
        };
    }, []);
    if (state === defaultStoreValue) {
        throw new NoStoreError(`No store context of "${name}" found. And "${name}" doesn't have defaultValue. Either render a Provider or set a defaultValue for this Store.`, s);
    }
    return state;
}
export function useStoreOptionally(s, deps) {
    try {
        const store = useStore(s, deps);
        return [store, true];
    }
    catch (e) {
        if (e instanceof NoStoreError) {
            return [undefined, false];
        }
        else {
            throw e;
        }
    }
}
function compare(oldDeps, newDeps) {
    if (oldDeps.length !== newDeps.length) {
        return true;
    }
    for (const index in newDeps) {
        if (oldDeps[index] !== newDeps[index]) {
            return true;
        }
    }
    return false;
}
